#
# Test of the debugger 'highlight' command
#
set basename on
set trace on
set skip off
show highlight
set highlight
set highlight off
set highlight plain
set highlight dark
set highlight light
quit!
quit!
